/***************************************************************
 * CSE455 Winter 2012 Project 3
 * File to be modified #1:
 * svmmath.cpp
 *		a routine for intersecting >2 lines (for vanishing point
 *		computation);
 *		routines for computing the homography for the reference
 *		plane and arbitrary polygons
 **************************************************************/

#pragma warning(disable : 4996)

#include "svmmath.h"
#include "jacob.h"
#include <cstring>
#include <cstdio>
#include <assert.h>

using namespace std;

/*
 * Helper function:
 * void MinEig(M, eval, evec, n)
 *		Given an n by n semi-positive-definite matrix M,
 *		returns the eigenvector for the smallest eigenvalue in evec.
 *		Will be useful for both line intersection and homography
 */	
void MinEig(double **M, double &eval, double *evec, long n)
{
	double **Nmatrix, **eigmatrix, *eigenvec, mineig;
	int nrot, minind, i,j;
	
	Nmatrix = nrmatrix(1,n,1,n);
	eigmatrix = nrmatrix(1,n,1,n);
	// allocate a float vector with subscript range v[1..n]
	eigenvec = nrvector(1,n);

	for (i=1; i<=n; i++)
	{
		for (j=1; j<=n; j++)
		{
			Nmatrix[i][j] = M[i][j];
			printf("%d %d \t %f\n", i, j, Nmatrix[i][j]);
		}
	}
	
	// run jacobi method to do eigen-decomposition 
	jacobi(Nmatrix, n, eigenvec, eigmatrix, &nrot);
	
	// find smallest eigenvalue 
	mineig = eigenvec[1];
	minind = 1;
	for (i=2; i <= n; i++) {
		if (eigenvec[i] < mineig) {
			mineig = eigenvec[i];
			minind = i;
		}
	}

	eval = mineig;
	//evec = Vec3d(eigmatrix[1][minind], eigmatrix[2][minind], eigmatrix[3][minind]);
	for (i=0; i<n; i++) {
		evec[i] = eigmatrix[i+1][minind];
	}
	
	// eigenvector associated with smallest eigenvalue put in x,y,z 
	printf("eigenvector associated with smallest eigenvalue:\n");
	for (i=1;i<=n;i++)
	{
		printf("%f\n",eigmatrix[i][minind]);
	}
	printf("\n");
	
	// free up allocated space 
	free_nrmatrix(Nmatrix,1,n,1,n);
	free_nrmatrix(eigmatrix,1,n,1,n);
	free_nrvector(eigenvec,1,n);
}

//
// TODO 1: BestFitIntersect()
//		Given lines, the list of 3 or more lines to be intersected,
//		find the best fit intersection point.
//		See http://www-2.cs.cmu.edu/~ph/869/www/notes/vanishing.txt.
//	
SVMPoint BestFitIntersect(const std::list<SVMLine> &lines, int imgWidth, int imgHeight)
{
	// check
	if (lines.size() < 2)
	{
		fprintf(stderr, "Not enough lines to compute the best fit.");
		abort();
	}

	SVMPoint bestfit;
	list<SVMLine>::const_iterator iter;

	// To accumulate stuff
	// CAREFUL: M's index starts from 1 to n, not from 0 to n-1 !!

	double **M = nrmatrix(1,3,1,3);
	int i, j;
	for (i=1; i<=3; i++)
	{
		for (j=1; j<=3; j++)
		{
			M[i][j] = 0;
		}
	}

	// Transformation for numerical stability

	// Note: iterate through the lines list as follows:
	//		for (iter = lines.begin(); iter != lines.end(); iter++) {
	//			...iter is the pointer to the current line...
	//		}
	// Note: Function to find eigenvector with smallest eigenvalue is MinEig(M, eval, evec, n)
	//   *** M is 1-index: M[1][1], M[1][2], ..., M[3][3].
	//   *** evec is 0-index: evec[0], evec[1], evec[2].
	//
	/******** BEGIN TODO ********/

	printf("BestFitIntersect() to be implemented!\n");

	/******** END TODO ********/
	
	// free up allocated space 
	free_nrmatrix(M,1,3,1,3);

	return bestfit;
}


//
// TODO 2: ConvertToPlaneCoordinate()
//		Given a plane defined by points, converts their coordinate into
//		the plane coordinate. See the following document for more detail.
//		http://www.cs.washington.edu/education/courses/cse455/08wi/projects/project3/homography.pdf.
//      The final divisors you apply to the u and v coordinates should be saved uScale and vScale
//
void ConvertToPlaneCoordinate(const vector<SVMPoint>& points, vector<Vec3d>& basisPts, double &uScale, double &vScale)
{
	/******** BEGIN TODO ********/

	printf("ConvertToPlaneCoordinate() to be implemented!\n");

	/******** END TODO ********/
}



//
// TODO 3: ComputeHomography()
//		Computes the homography H from the plane specified by "points" to the image plane,
//		and its inverse Hinv.
//		If the plane is the reference plane (isRefPlane == true), don't convert the
//		coordinate system to the plane. Only do this for polygon patches where
//		texture mapping is necessary.
//		Coordinate system conversion is to be implemented in a separate routine
//		ConvertToPlaneCoordinate.
//		For more detailed explaination, see
//		http://www.cs.washington.edu/education/courses/cse455/08wi/projects/project3/homography.pdf.
//
void ComputeHomography(double H[3][3], double Hinv[3][3], const vector<SVMPoint>& points, vector<Vec3d> &basisPts, bool isRefPlane)
{
	int i;
	assert( points.size() >= 4 );

	basisPts.clear();
	if (isRefPlane) // reference plane
	{
		for (i=0; i<(int)points.size(); i++)
		{
			Vec3d tmp = Vec3d(points[i].X, points[i].Y, points[i].Z);
			basisPts.push_back(tmp);
		}
	}
	else // arbitrary polygon
	{
        double uScale, vScale; // unused in this function
		ConvertToPlaneCoordinate(points, basisPts, uScale, vScale);
	}

	// A: 2n x 9 matrix where n is the number of points on the plane
	//    as discussed in lecture note
	// A_squared: A^T * A
	double **A = nrmatrix(1,2*points.size(),1,9);
	double **A_squared = nrmatrix(1, 9, 1, 9);
	// CAREFUL: for A and A_squared indices are from 1 to n, instead of 0 to n-1 !!



	/******** BEGIN TODO ********/

	printf("ComputeHomography() to be implemented!\n");

	/******** END TODO ********/



	// compute inverse of H
	if (Determinant(H) == 0)
		fl_alert("Computed homography matrix is uninvertible \n");
	else
		Matrix3by3Inv(Hinv,H);

	int ii;
	printf("\nH=[\n");
	for (ii=0; ii<3; ii++)
		printf("%e\t%e\t%e;\n", H[ii][0]/H[2][2], H[ii][1]/H[2][2], H[ii][2]/H[2][2]);
	printf("]\nHinv=[\n");

	for (ii=0; ii<3; ii++)
		printf("%e\t%e\t%e;\n", Hinv[ii][0]/Hinv[2][2], Hinv[ii][1]/Hinv[2][2], Hinv[ii][2]/Hinv[2][2]);

	printf("]\n\n");

	// free up allocated space 
	free_nrmatrix(A,1,2*points.size(),1,9);
	free_nrmatrix(A_squared,1,9,1,9);
}
